--		ActivityMan:StartActivity("GAScripted", "Hammer Canyon")

function HammerCanyon:StartActivity()
	self.KeyTimer = Timer()
	self.SkillTimer = Timer()
	
	-- Set up players
	if self.PresetName == "Hammer Canyon" then
		self.HammerTime = true
		self:SetTeamFunds(0, Activity.TEAM_1)
		for player = 0, self.PlayerCount - 1 do
			local Brain = CreateActor("Brain Case")
			if Brain then
				Brain.Pos = Vector(-70 + player * 7, -70 + player * 7)
				self:SetPlayerBrain(Brain, player)
				self:SetObservationTarget(Brain.Pos, player)
				MovableMan:AddActor(Brain)
				
				local name
				local rand = SelectRand(1,4)
				if rand == 1 then
					name = "Hammer Ship"
				elseif rand == 2 then
					name = "Mace Ship"
				else
					name = "Ball and Chain Ship"
				end
				
				self.Hammer = CreateACDropShip(name, "Hammer.rte")
				if self.Hammer then
					self.Hammer.Team = player
					self.Hammer.AIMode = Actor.AIMODE_STAY
					self.Hammer.Pos = Vector(RangeRand(900, 1100), -60)
					self.Hammer.Vel.Y = 20
					self.Hammer:ReloadScripts()
					MovableMan:AddActor(self.Hammer)
					self:SwitchToActor(self.Hammer, self.Hammer.Team, self.Hammer.Team)
				end
			end
		end
	else
		self:SetTeamFunds(1000, Activity.TEAM_1)
		for player = 0, self.PlayerCount - 1 do
			-- Check if we already have a brain assigned
			if not self:GetPlayerBrain(player) then
				self.ActivityState = Activity.EDITING
			end
		end
	end
	
	self.Exit = SceneMan.Scene:GetArea("Exit"):GetCenterPoint()
	
	-- Spawn
	self.DropShips = {}
	self.Spawn = {min = 4500, max = 5500}
	
	-- Triggers
	self.TriggerTimer = Timer()
	self.Triggers = {}
	
	local NewTrigger = {}
	NewTrigger.Timer = Timer()
	NewTrigger.Result = LandEnemyTroops
	
	table.insert(self.Triggers, NewTrigger)
	
	-- Globals
	gHammerRockets = 0
	gHammerShips = 0
	gHammerRobots = 0
	gHammerPoints = 0
end

function LandEnemyTroops(self, Trigger)
	if MovableMan:GetMOIDCount() < 100 then
		local ShipsX = {}
		local Remove = {}
		for i, S in ipairs(self.DropShips) do
			if MovableMan:IsActor(S) and S.ClassName ~= "Entity" then
				table.insert(ShipsX, {x=S.Pos.X, w=S.Diameter/2+75})
			else
				table.insert(Remove, i)
			end
		end
		
		for i = 1, #Remove do
			self.DropShips[i] = nil
		end
		
		-- try to find a place to land
		local posX
		local start = 400
		local stop = 1800
		if #ShipsX > 0 then
			local testX = nil
			for i = 1, 4 do	-- try four times
				testX = RangeRand(start, stop)
				for i, S in ipairs(ShipsX) do
					if S.x + S.w < testX - 125 or S.x - S.w > testX + 125 then
						posX = testX
					else
						posX = nil
						break
					end
				end
				
				if posX then
					break
				end
			end
		else
			posX = RangeRand(start, stop)
		end
		
		if posX then	-- LZ found
			Trigger.nextTimerLimit = SelectRand(self.Spawn.min, self.Spawn.max)
			
			local Ship
			if PosRand() > 0.4 then
				Ship = CreateACDropShip("Drop Ship MK1b", "Hammer.rte")
			else
				Ship = CreateACRocket("Rocket MK2b", "Hammer.rte")
			end
			
			if Ship then
				local Act
				if PosRand() < 0.4 then
					Act = CreateAHuman("Robot 2b", "Hammer.rte")
					if Act then
						Act:AddInventoryItem(CreateHDFirearm("Pistol", "Base.rte"))
					end
				else
					Act = CreateAHuman("Robot 1b", "Hammer.rte")
				end
				
				if Act then
					Act.Team = Activity.TEAM_2
					Act.AIMode = Actor.AIMODE_GOTO
					Act:AddAISceneWaypoint(self.Exit)
					Ship:AddInventoryItem(Act)
				end
				
				Ship.Team = Activity.TEAM_2
				Ship.Pos = Vector(posX, -90)
				table.insert(self.DropShips, Ship)
				MovableMan:AddActor(Ship)
			end
		else
			Trigger.nextTimerLimit = SelectRand(self.Spawn.min/2, self.Spawn.max/2)
		end
	end
end

function HammerCanyon:UpdateActivity()
	if self.ActivityState == Activity.EDITING then
		return
	end
	
	local p = gHammerRockets * 4 + gHammerRobots * 5 + gHammerShips * 2
	local e
	
	if self.HammerTime then
		e = gHammerPoints * 5
	else
		e = gHammerPoints * 10
	end
	
	if self.ActivityState == Activity.OVER then
		FrameMan:SetScreenText("Player: " .. p .. "   Enemy: " .. e, 0, 0, 1, true)
		return
	elseif self.HammerTime then
		if not self.Hammer then
			FrameMan:SetScreenText("Wait for reinforcements", 0, 0, 1, true)
		elseif not MovableMan:ValidMO(self.Hammer) or self.Hammer.ClassName == "Entity" then
			self.Hammer = nil
			self.SkillTimer:Reset()
			gHammerPoints = gHammerPoints + 5
		end
	end
	
	FrameMan:SetScreenText("Player: " .. p .. "   Enemy: " .. e, 0, 0, 1, false)
	
	local min = 300
	local range
	for Act in MovableMan.Actors do
		if Act.Team == Activity.TEAM_2 and Act.ClassName == "AHuman" then
			range = Act.Pos.X
			if min > range and range > 0 then
				min = range
				self.Marker = Act
			end
		end
	end
	
	self:ClearObjectivePoints()
	if self.Marker then
		if MovableMan:IsActor(self.Marker) and self.Marker.ClassName ~= "Entity" then
			self:AddObjectivePoint("Stop him!", self.Marker.AboveHUDPos, Activity.TEAM_1, GameActivity.ARROWDOWN)
		else
			self.Marker = nil
		end
	end
	
	-- Triggers
	if self.TriggerTimer:IsPastSimMS(600) then
		self.TriggerTimer:Reset()
		
		for i,Trg in next, self.Triggers do
			if Trg.Timer:IsPastSimMS(Trg.timerLimit or 0) then
				Trg.Timer:Reset()
				Trg.Result(self, Trg)
				Trg.timerLimit = Trg.nextTimerLimit
				break
			end
		end
	end
	
	if self.SkillTimer:IsPastSimMS(10000) then
		self.SkillTimer:Reset()
		
		if self.HammerTime and not self.Hammer then
			local name
			local rand = SelectRand(1,4)
			if rand == 1 then
				name = "Hammer Ship"
			elseif rand == 2 then
				name = "Mace Ship"
			else
				name = "Ball and Chain Ship"
			end
			
			self.Hammer = CreateACDropShip(name, "Hammer.rte")
			if self.Hammer then
				self.Hammer.Team = Activity.TEAM_1
				self.Hammer.AIMode = Actor.AIMODE_STAY
				self.Hammer.Pos = Vector(RangeRand(900, 1100), -60)
				self.Hammer.Vel.Y = 20
				self.Hammer:ReloadScripts()
				MovableMan:AddActor(self.Hammer)
				self:SwitchToActor(self.Hammer, self.Hammer.Team, self.Hammer.Team)
			end
		end
		
		if p > e then
			self.Spawn.min = math.max(self.Spawn.min - 200, 1000)
			self.Spawn.max = math.max(self.Spawn.max - 100, 2000)
		elseif p < e/2 then
			self.Spawn.min = self.Spawn.min + 100
			self.Spawn.max = self.Spawn.max + 50
		end
	end
	
	-- Debug spawn
	if self.KeyTimer:IsPastSimMS(150) and UInputMan:KeyHeld(71) then
		if UInputMan:KeyHeld(15) then	-- O
			self.KeyTimer:Reset()
			
			local Act = CreateAHuman("Robot 1", "Base.rte")
			if Act then
				Act.Team = Activity.TEAM_1
				Act.AIMode = Actor.AIMODE_SENTRY
				Act.Pos = SceneMan:MovePointToGround(Vector(1200, 100), 21, 5)
				Act:AddInventoryItem(CreateHDFirearm("AK-47", "Base.rte"))
				MovableMan:AddActor(Act)
				self:SwitchToActor(Act, Act.Team, Act.Team)
			end
		elseif UInputMan:KeyHeld(16) then	-- P
			self.KeyTimer:Reset()
			
			local Act = CreateAHuman("Robot 2", "Base.rte")
			if Act then
				Act.Team = Activity.TEAM_2
				Act.AIMode = Actor.AIMODE_SENTRY
				Act.Pos = SceneMan:MovePointToGround(Vector(RangeRand(400, 600), 0), 21, 5)
				Act.HFlipped = false
				MovableMan:AddActor(Act)
			end
			
			Act = CreateAHuman("Robot 1", "Base.rte")
			if Act then
				Act.Team = Activity.TEAM_2
				Act.AIMode = Actor.AIMODE_SENTRY
				Act.Pos = SceneMan:MovePointToGround(Vector(RangeRand(700, 900), 0), 21, 5)
				MovableMan:AddActor(Act)
			end
		elseif UInputMan:KeyHeld(18) then	-- R
			self.KeyTimer:Reset()
			local Act = CreateACDropShip("Ball and Chain Ship", "Hammer.rte")
			if Act then
				Act.Team = Activity.TEAM_1
				Act.AIMode = Actor.AIMODE_STAY
				Act.Pos = Vector(RangeRand(200, 600), 250)
				print(Act.PresetName .. " reload = " .. Act:ReloadScripts())
				MovableMan:AddActor(Act)
				self:SwitchToActor(Act, Act.Team, Act.Team)
			end
		elseif UInputMan:KeyHeld(22) then	-- V
			self.KeyTimer:Reset()
			self:SetViewState(Activity.ACTORSELECT, Activity.TEAM_1)
		elseif UInputMan:KeyHeld(24) then	-- X
			self.KeyTimer:Reset()
			for Act in MovableMan.Actors do
				if Act.PresetName ~= "Brain Robot" and Act.PresetName ~= "Brain Case" and Act.ClassName ~= "ADoor" then
					Act:GibThis()
				end
			end
		elseif UInputMan:KeyHeld(26) then	-- Z
			self.KeyTimer:Reset()
			for Act in MovableMan.Actors do
				if Act.PresetName ~= "Brain Robot" and Act.PresetName ~= "Brain Case" and Act.ClassName ~= "ADoor" then
					Act.ToDelete = true
				end
			end
			
			for Pt in MovableMan.Particles do
				Pt.ToDelete = true
			end
		end
	end
	
	-- Record screenshots
	if gGIF and self.ScreenShotTimer:IsPastRealMS(66) then	-- 42=24fps
		self.ScreenShotTimer:Reset()
		FrameMan:SaveScreenToBMP(self.fileName)
	end
end